function [ rmse, time_record ] = BMC_ADMM( train, test, minRating, maxRating, t, k, max_iter, flag, ini)
%BMC_ADMM Summary of this function goes here
%   Detailed explanation goes here
%   train: training data, a sparse matrix
%   test: testing data, a sparse matrix
%   minRating, maxRating: lower and upper bound of ratings
%   lambda: trade off between appoximation and low-rank penalty.
%   k: reduced rank k
%   flag: "obj" to output objective value, "test" to output test error.
%   ini: 0 for random initialization, 1 for baseline initialization(implemented by BMA)
clear global;
global A1 A2 Sparse_S;
rho = 1;
addpath PROPACK;

rmse = [];
time_record = [];
Y = train;
[m, n] = size(Y);
d = min(m, n);

% same initialization as BMA
if(ini == 1),
    [P, Q] = decomposebaseline(Y, k, minRating, maxRating);
else
    P = single(rand(m,k));
    Q = single(rand(k,n));
    maxElement = -inf;
    for x = 1:m
        v = P(x,2:end)*Q(2:end,:);
        if (maxElement < max(v))
            maxElement = max(v);
        end
    end
    normalizingConstant = sqrt((maxRating-1)/maxElement);
    
    P = normalizingConstant*P;
    Q = normalizingConstant*Q;
    P (:,1) = 1;
    Q (1,:) = 1;
end

A1 = double(P);
A2 = double(Q');

U1 = sparse(m, n);
U2 = sparse(m, n);

R = step2(A1, A2, U2, minRating, maxRating);
R = setzeros(R, U2, m, n, nnz(R));
R_tmp = getR(Y, U2, A1, A2, minRating, maxRating, nnz(U2), nnz(Y));
R = R + R_tmp;

converge = false;
iter = 1;
tic
while(~converge),
    %tic
    Sparse_A1A2 = step1(Y,A1,A2,nnz(Y));
    X = (Y + rho*(Sparse_A1A2 - U1)) / (1+rho);

    Sparse_S = 0.5*(X - Sparse_A1A2 + U1 + R - U2);
    [A1,S,A2] = lansvd('Axz','Atxz',m,n,k,'L');
    %toc
    
    diagS = diag(S);
    svp = length(find(diagS > t/(rho*2)));
    sqrtds = sqrt(diagS(1:svp) - t/(rho*2));
    A1 = A1(:,1:svp) * diag(sqrtds);
    A2 = A2(:,1:svp) * diag(sqrtds);

    %tic
    R = step2(A1, A2, U2, minRating, maxRating);
    
    
    R = setzeros(R, U2, m, n, nnz(R));
    R_tmp = getR(Y, U2, A1, A2, minRating, maxRating, nnz(U2), nnz(Y));
    R = R + R_tmp;

    %toc
    U1 = U1 + X - Sparse_A1A2;
    U2 = U2 - R;
    
    %tic
    err = evalRMSE(test, A1, A2, nnz(test));
    
    if strcmp(flag,'obj'),
        obj = evalRMSE(train, A1, A2, nnz(train))^2*nnz(train)/2 + t*sum(diagS(1:svp));
        rmse(end+1) = obj;
    else
        rmse(end+1) = err;
    end

    fprintf('evaluation: ');
    %toc
    time_record(end + 1) = toc;
    iter = iter + 1;
    fprintf('# iter: %i, test RMSE: %f\n', iter, err);
    if iter >= max_iter,
        converge = true;
    end
end
end

