#include <math.h> /* Needed for the ceil() prototype. */
#include "mex.h"
#include <vector>
#include <omp.h>
using namespace std;

void mexFunction(
        int nlhs,       mxArray *plhs[],
        int nrhs, const mxArray *prhs[]
        )
{ 
  
  /*input A1, A2, U2, r1, r2 */
  /*output B(A1*A2') - A1*A2' */
  double *A1, *A2, *U2;
  double r1, r2;
  mwIndex *o_irs, *o_jcs;
  double *o_rrs;
  int m, n, k;  
  

  A1 = mxGetPr(prhs[0]);
  A2 = mxGetPr(prhs[1]);
  U2 = mxGetPr(prhs[2]);
  r1 = (double) mxGetScalar(prhs[3]);
  r2 = (double) mxGetScalar(prhs[4]);
  
  //mexPrintf("r1: %f r2: %f", r1, r2);
  m = mxGetM(prhs[0]);
  k = mxGetN(prhs[0]);
  n = mxGetM(prhs[1]);

  int i, j, q;
  int count = 0;
  vector<int> idx_row;
  vector<int> idx_col;
  vector<double> sp_val;
  double val;
#pragma omp parallel for
  for(j = 0; j < n; j++){
    for(i = 0; i < m; i++){
      val = 0;
      for(q = 0; q < k; q++){
        val += A1[i + m*q] * A2[j + n*q];
      }
      //mexPrintf("val: %f\n", val);
      if(val > r2){
        //mexPrintf(">r2\n");
        count++;
        idx_row.push_back( i );
        idx_col.push_back( j );
        sp_val.push_back( r2 - val );
      }
      if(val < r1){
        count++;
        idx_row.push_back( i );
        idx_col.push_back( j );
        sp_val.push_back( r1 - val );
      }
    }
  }

  //mexPrintf("count: %d", count);
  plhs[0] = mxCreateSparse(m, n, count, mxREAL);
  o_irs = mxGetIr(plhs[0]);
  o_jcs = mxGetJc(plhs[0]);
  o_rrs = mxGetPr(plhs[0]);
  int n_col = 0;
  o_jcs[0] = 0;
  int pre = 0;
  for(i = 0; i < count; i++){
    o_irs[i] = idx_row[i];
    o_rrs[i] = sp_val[i];
    //mexPrintf("idx_row: %i\n", idx_row[i]);
    //mexPrintf("sp_val: %f\n", sp_val[i]);
    if(idx_col[i] > pre) {
	  o_jcs[pre + 1] = i;
	  for(n_col = pre + 1; n_col < idx_col[i]; n_col++){
		o_jcs[n_col + 1] = o_jcs[n_col];
	  }
	  pre = idx_col[i];
    }
  }
  o_jcs[pre + 1] = count;
  for(n_col = pre + 1; n_col < n; n_col++){
	  o_jcs[n_col + 1] = o_jcs[n_col];
  }

}
